@@ -22,6 +22,10 @@ All device-specific handlers derive from the DefaultDeviceHandler, which impleme
 generic information needed for interaction with a Netconf server.
 
 """
+import sys
+if sys.version >= '3':
+    xrange = range
+
 class DefaultDeviceHandler(object):
     """
     Default handler for device specific information.
@@ -87,6 +91,7 @@ class DefaultDeviceHandler(object):
             "urn:ietf:params:netconf:capability:url:1.0?scheme=http,ftp,file,https,sftp",
             "urn:ietf:params:netconf:capability:validate:1.0",
             "urn:ietf:params:netconf:capability:xpath:1.0",
+            "urn:ietf:params:netconf:capability:notification:1.0",
             "urn:liberouter:params:netconf:capability:power-control:1.0",
             "urn:ietf:params:netconf:capability:interleave:1.0"
         ]
