@@ -12,9 +12,13 @@ generic information needed for interaction with a Netconf server.
 
 """
 
+import re
+from lxml import etree
 from .default import DefaultDeviceHandler
 from ncclient.operations.third_party.juniper.rpc import GetConfiguration, LoadConfiguration, CompareConfiguration
 from ncclient.operations.third_party.juniper.rpc import ExecuteRpc, Command, Reboot, Halt, Commit
+from ncclient.operations.rpc import RPCError
+from ncclient.xml_ import to_ele
 
 class JunosDeviceHandler(DefaultDeviceHandler):
     """
@@ -41,8 +45,29 @@ class JunosDeviceHandler(DefaultDeviceHandler):
 
     def handle_raw_dispatch(self, raw):
         if 'routing-engine' in raw:
-                raw = re.sub(r'<ok/>', '</routing-engine>\n<ok/>', raw)
-                return raw
+            raw = re.sub(r'<ok/>', '</routing-engine>\n<ok/>', raw)
+            return raw
+        # check if error is during cpapbilites exchange itself
+        elif re.search('\<rpc-reply\>.*?\</rpc-reply\>.*\</hello\>?', raw, re.M|re.S):
+            errs = re.findall('\<rpc-error\>.*?\</rpc-error\>', raw, re.M|re.S)
+            err_list = []
+            if errs:
+                add_ns = """
+                        <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+                          <xsl:output indent="yes"/>
+                            <xsl:template match="*">
+                            <xsl:element name="{local-name()}" namespace="urn:ietf:params:xml:ns:netconf:base:1.0">
+                            <xsl:apply-templates select="@*|node()"/>
+                            </xsl:element>
+                          </xsl:template>
+                        </xsl:stylesheet>"""
+                for err in errs:
+                    doc = etree.ElementTree(etree.XML(err))
+                    # Adding namespace using xslt
+                    xslt = etree.XSLT(etree.XML(add_ns))
+                    transformed_xml = etree.XML(etree.tostring(xslt(doc)))
+                    err_list.append(RPCError(transformed_xml))
+                return RPCError(to_ele(''.join(errs)), err_list)
         else:
             return False
 
@@ -53,7 +78,7 @@ class JunosDeviceHandler(DefaultDeviceHandler):
         return True
     
     def transform_reply(self):
-        return '''<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+        reply = '''<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
         <xsl:output method="xml" indent="no"/>
 
         <xsl:template match="/|comment()|processing-instruction()">
@@ -75,3 +100,8 @@ class JunosDeviceHandler(DefaultDeviceHandler):
         </xsl:template>
         </xsl:stylesheet>
         '''
+        import sys
+        if sys.version < '3':
+            return reply
+        else:
+            return reply.encode('UTF-8')
