@@ -18,10 +18,10 @@ This module is a thin layer of abstraction around the library.
 It exposes all core functionality.
 """
 
-import capabilities
-import operations
-import transport
-
+from ncclient import capabilities
+from ncclient import operations
+from ncclient import transport
+import six
 import logging
 
 from ncclient.xml_ import *
@@ -41,6 +41,7 @@ OPERATIONS = {
     "delete_config": operations.DeleteConfig,
     "lock": operations.Lock,
     "unlock": operations.Unlock,
+    "create_subscription": operations.CreateSubscription,
     "close_session": operations.CloseSession,
     "kill_session": operations.KillSession,
     "poweroff_machine": operations.PoweroffMachine,
@@ -145,10 +146,13 @@ def connect_ioproc(*args, **kwds):
 def connect(*args, **kwds):
     if "host" in kwds:
         host = kwds["host"]
-        if host != 'localhost':
-            return connect_ssh(*args, **kwds)
-        else:
+        device_params = kwds.get('device_params', {})
+        if host == 'localhost' and device_params.get('name') == 'junos' \
+                and device_params.get('local'):
             return connect_ioproc(*args, **kwds)
+        else:
+            return connect_ssh(*args, **kwds)
+
 
 class OpExecutor(type):
 
@@ -156,9 +160,9 @@ class OpExecutor(type):
         def make_wrapper(op_cls):
             def wrapper(self, *args, **kwds):
                 return self.execute(op_cls, *args, **kwds)
-            wrapper.func_doc = op_cls.request.func_doc
+            wrapper.__doc__ = op_cls.request.__doc__
             return wrapper
-        for op_name, op_cls in OPERATIONS.iteritems():
+        for op_name, op_cls in six.iteritems(OPERATIONS):
             attrs[op_name] = make_wrapper(op_cls)
         return super(OpExecutor, cls).__new__(cls, name, bases, attrs)
 
@@ -166,15 +170,15 @@ class OpExecutor(type):
         def make_wrapper(op_cls):
             def wrapper(self, *args, **kwds):
                 return self.execute(op_cls, *args, **kwds)
-            wrapper.func_doc = op_cls.request.func_doc
+            wrapper.__doc__ = op_cls.request.__doc__
             return wrapper
         if VENDOR_OPERATIONS:
-            for op_name, op_cls in VENDOR_OPERATIONS.iteritems():
+            for op_name, op_cls in six.iteritems(VENDOR_OPERATIONS):
                 setattr(cls, op_name, make_wrapper(op_cls))
         return super(OpExecutor, cls).__call__(*args, **kwargs)
 
 
-class Manager(object):
+class Manager(six.with_metaclass(OpExecutor, object)):
 
     """
     For details on the expected behavior of the operations and their
@@ -194,7 +198,7 @@ class Manager(object):
             m.close_session()
     """
 
-    __metaclass__ = OpExecutor
+   # __metaclass__ = OpExecutor
 
     def __init__(self, session, device_handler, timeout=30, *args, **kwargs):
         self._session = session
@@ -262,6 +266,24 @@ class Manager(object):
             return r
         return _missing
 
+    def take_notification(self, block=True, timeout=None):
+        """Attempt to retrieve one notification from the queue of received
+        notifications.
+
+        If block is True, the call will wait until a notification is
+        received.
+
+        If timeout is a number greater than 0, the call will wait that
+        many seconds to receive a notification before timing out.
+
+        If there is no notification available when block is False or
+        when the timeout has elapse, None will be returned.
+
+        Otherwise a :class:`~ncclient.operations.notify.Notification`
+        object will be returned.
+        """
+        return self._session.take_notification(block, timeout)
+
     @property
     def client_capabilities(self):
         """:class:`~ncclient.capabilities.Capabilities` object representing
