@@ -13,12 +13,13 @@
 # limitations under the License.
 
 from threading import Event, Lock
-from uuid import uuid1
+from uuid import uuid4
+import six
 
 from ncclient.xml_ import *
 from ncclient.transport import SessionListener
 
-from errors import OperationError, TimeoutExpiredError, MissingCapabilityError
+from ncclient.operations.errors import OperationError, TimeoutExpiredError, MissingCapabilityError
 
 import logging
 logger = logging.getLogger("ncclient.operations.rpc")
@@ -41,7 +42,7 @@ class RPCError(OperationError):
         self._raw = raw
         if errs is None:
             # Single RPCError
-            for attr in RPCError.tag_to_attr.values():
+            for attr in six.itervalues(RPCError.tag_to_attr):
                 setattr(self, attr, None)
             for subele in raw:
                 attr = RPCError.tag_to_attr.get(subele.tag, None)
@@ -75,7 +76,7 @@ class RPCError(OperationError):
             OperationError.__init__(self, self.message)
 
     def to_dict(self):
-        return dict([ (attr[1:], getattr(self, attr)) for attr in RPCError.tag_to_attr.values() ])
+        return dict([ (attr[1:], getattr(self, attr)) for attr in six.itervalues(RPCError.tag_to_attr) ])
 
     @property
     def xml(self):
@@ -216,7 +217,7 @@ class RPCReplyListener(SessionListener): # internal use
                         logger.debug("Delivering to %r" % rpc)
                         rpc.deliver_reply(raw)
                     except KeyError:
-                        raise OperationError("Unknown 'message-id': %s", id)
+                        raise OperationError("Unknown 'message-id': %s" % id)
                     # no catching other exceptions, fail loudly if must
                     else:
                         # if no error delivering, can del the reference to the RPC
@@ -227,7 +228,7 @@ class RPCReplyListener(SessionListener): # internal use
 
     def errback(self, err):
         try:
-            for rpc in self._id2rpc.values():
+            for rpc in six.itervalues(self._id2rpc):
                 rpc.deliver_error(err)
         finally:
             self._id2rpc.clear()
@@ -284,7 +285,7 @@ class RPC(object):
         self._async = async
         self._timeout = timeout
         self._raise_mode = raise_mode
-        self._id = uuid1().urn # Keeps things simple instead of having a class attr with running ID that has to be locked
+        self._id = uuid4().urn # Keeps things simple instead of having a class attr with running ID that has to be locked
         self._listener = RPCReplyListener(session, device_handler)
         self._listener.register(self._id, self)
         self._reply = None
